@@ -23,6 +23,8 @@
 #include <limits.h>
 #include <math.h>
 #include <time.h>
+#include <stdlib.h>
+#include <sys/time.h>
 #include "jalali.h"
 #include "jconfig.h"
 
@@ -111,8 +113,20 @@ jalali_is_jleap(int year)
 void
 jalali_create_time_from_secs(time_t t, struct ab_jtm* d)
 {
-	d->ab_days = (t / J_DAY_LENGTH_IN_SECONDS);
-	t %= J_DAY_LENGTH_IN_SECONDS;
+	d->ab_days = (t >= 0) ? (t / (time_t) J_DAY_LENGTH_IN_SECONDS) :
+		((t - (time_t) J_DAY_LENGTH_IN_SECONDS + (time_t) 1) / 
+		 (time_t) J_DAY_LENGTH_IN_SECONDS);
+
+	if (t >= 0) {
+		t %= (time_t) J_DAY_LENGTH_IN_SECONDS;
+	}
+	else {
+		t = (J_DAY_LENGTH_IN_SECONDS - 
+			 (abs(t - J_DAY_LENGTH_IN_SECONDS) %
+			  J_DAY_LENGTH_IN_SECONDS)) %
+			J_DAY_LENGTH_IN_SECONDS;
+	}
+
 	d->ab_hour = t / J_HOUR_LENGTH_IN_SECONDS;
 	t %= J_HOUR_LENGTH_IN_SECONDS;
 	d->ab_min = t / J_MINUTE_LENGTH_IN_SECONDS;
@@ -126,10 +140,11 @@ jalali_create_time_from_secs(time_t t, struct ab_jtm* d)
 time_t
 jalali_create_secs_from_time(const struct ab_jtm* d)
 {
-	return (d->ab_days * J_DAY_LENGTH_IN_SECONDS +
-			d->ab_hour * J_HOUR_LENGTH_IN_SECONDS +
-			d->ab_min * J_MINUTE_LENGTH_IN_SECONDS +
-			d->ab_sec);
+	return (time_t)
+		((time_t) d->ab_days * (time_t) J_DAY_LENGTH_IN_SECONDS +
+		 (time_t) d->ab_hour * (time_t) J_HOUR_LENGTH_IN_SECONDS +
+		 (time_t) d->ab_min * (time_t) J_MINUTE_LENGTH_IN_SECONDS +
+		 (time_t) d->ab_sec);
 }
 
 /*
@@ -235,6 +250,9 @@ jalali_get_date(int p, struct jtm* j)
 {
 	time_t t;
 	struct tm lt;
+	struct timezone tz;
+	struct timeval tv;
+
 	int wd = (p + J_UTC_EPOCH_WDAY) % J_WEEK_LENGTH;
 
 	if (wd < 0) {
@@ -267,7 +285,10 @@ jalali_get_date(int p, struct jtm* j)
 	tzset();
 	t = p * J_DAY_LENGTH_IN_SECONDS;
 	localtime_r(&t, &lt);
-	j->tm_gmtoff = lt.tm_gmtoff;
+	gettimeofday(&tv, &tz);
+
+	j->tm_gmtoff = (-tz.tz_minuteswest) * J_MINUTE_LENGTH_IN_SECONDS
+		+ (tz.tz_dsttime * J_HOUR_LENGTH_IN_SECONDS);
 	j->tm_isdst = lt.tm_isdst;
 	j->tm_zone = tzname[lt.tm_isdst];
 }
