@@ -22,13 +22,13 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-
+#include <sys/time.h>
 #include "jconfig.h"
 #include "jalali.h"
 #include "jtime.h"
 
 const char* GMT_ZONE = "UTC";
-
+const char* GMT_ZONE_fa = "گرینویچ";
 const char* jalali_months[] = { "Farvardin", "Ordibehesht", "Khordaad",
 								"Tir", "Mordaad", "Shahrivar", "Mehr",
 								"Aabaan", "Aazar", "Dey", "Bahman", "Esfand" };
@@ -47,7 +47,7 @@ const char* fa_jalali_days[] = { "شنبه", "یکشنبه", "دوشنبه", "س
 								 "چهارشنبه", "پنجشنبه", "جمعه" };
 const char* jalali_days_3_fa[] = { "Sha", "Yek", "Dos", "Ses", "Cha", "Pan",
 									"Jom" };
-const char* fa_jalali_days_3[] = { "شنب", "یکش", "دوش", "سه ", "چها", "پنج",
+const char* fa_jalali_days_3[] = { "شنب", "یکش", "دوش", "سهش", "چها", "پنج",
 								   "جمع" };
 const char* jalali_days_2_fa[] = { "Sh", "Ye", "Do", "Se", "Ch", "Pa", "Jo" };
 
@@ -57,6 +57,10 @@ const char* jalali_days[] = { "Saturday", "Sunday", "Monday", "Tuesday",
 const char* jalali_days_3[] = { "Sat", "Sun", "Mon", "Tue", "Wed", "Thu",
 								"Fri" };
 const char* jalali_days_2[] = { "Sa", "Su", "Mo", "Tu", "We", "Th", "Fr" };
+const char* farsi_digits[] = { "۰", "۱", "۲", "۳", "۴",
+							   "۵", "۶", "۷", "۸", "۹" };
+
+const char* tzname_fa[2] = { "زمان زمستانی", "زمان تابستانی" };
 
 static char in_buf[MAX_BUF_SIZE] = {0};
 static struct jtm in_jtm;
@@ -100,11 +104,19 @@ in_jlocaltime(const time_t* timep, struct jtm* result) {
 	struct tm t;
 	struct jtm c_jtm;
 	struct ab_jtm ab;
+	struct timeval tv;
+	struct timezone tz;
+
+	long int gmtoff;
 	time_t c;
+	
+	gettimeofday(&tv, &tz);
+	gmtoff = (-tz.tz_minuteswest) * J_MINUTE_LENGTH_IN_SECONDS +
+		(tz.tz_dsttime * J_HOUR_LENGTH_IN_SECONDS);
 	tzset();
 
 	localtime_r(timep, &t);
-	c = (*timep) + t.tm_gmtoff;
+	c = (*timep) + (time_t) gmtoff;
 
 	jalali_create_time_from_secs(c, &ab);
 	jalali_get_date(ab.ab_days, &c_jtm);
@@ -115,7 +127,7 @@ in_jlocaltime(const time_t* timep, struct jtm* result) {
 	c_jtm.tm_isdst = t.tm_isdst;
 
 	c_jtm.tm_zone = tzname[t.tm_isdst];
-	c_jtm.tm_gmtoff = t.tm_gmtoff;
+	c_jtm.tm_gmtoff = gmtoff;
 
 	if (result) {
 		memcpy(result, &c_jtm, sizeof(struct jtm));
@@ -215,10 +227,11 @@ jmktime(const struct jtm* jtm) {
 
 	int p = jalali_get_diff(jtm);
 	time_t t;
-	t = (p * J_DAY_LENGTH_IN_SECONDS) +
-		(jtm->tm_hour * J_HOUR_LENGTH_IN_SECONDS)
-		+ (jtm->tm_min * J_MINUTE_LENGTH_IN_SECONDS) + jtm->tm_sec
-		- (jtm->tm_isdst * J_HOUR_LENGTH_IN_SECONDS) - (jtm->tm_gmtoff);
+	t = ((time_t) p * (time_t) J_DAY_LENGTH_IN_SECONDS) +
+		((time_t) jtm->tm_hour * (time_t) J_HOUR_LENGTH_IN_SECONDS)
+		+ ((time_t) jtm->tm_min * 
+		   (time_t) J_MINUTE_LENGTH_IN_SECONDS) + (time_t) jtm->tm_sec -
+		((time_t) jtm->tm_gmtoff);
 	return t;
 }
 
@@ -227,6 +240,11 @@ jstrftime(char* s, size_t max, const char* format, const struct jtm* jtm) {
 	if (!s || max <= 0 || !format || !jtm)
 		return -1;
 
+	char _l1[10];
+	char _l2[10];
+	char _l3[10];
+	char _la[100];
+	char _lb[100];
 	char buf[MAX_BUF_SIZE];
 	int i, j;
 	int fmt_n = strlen(format);
@@ -273,8 +291,8 @@ jstrftime(char* s, size_t max, const char* format, const struct jtm* jtm) {
 				snprintf(buf, MAX_BUF_SIZE, "%s %d %s %d %02d:%02d:%02d %s",
 						 jalali_days_3[jtm->tm_wday], jtm->tm_mday,
 						 jalali_months_3[jtm->tm_mon], jtm->tm_year,
-						 jtm->tm_hour,jtm->tm_min, jtm->tm_sec,
-						 tzname[jtm->tm_isdst]);
+						 jtm->tm_hour, jtm->tm_min, jtm->tm_sec,
+						 jtm->tm_zone);
 				break;
 
 				/* The century number (year/100) as a 2-digit integer. */
@@ -301,42 +319,60 @@ jstrftime(char* s, size_t max, const char* format, const struct jtm* jtm) {
 				snprintf(buf, MAX_BUF_SIZE, "%2d", jtm->tm_mday);
 				break;
 
-				/* Unused. */
+				/*
+				 * The preferred date and time representation in Farsi. (utf8)
+				 * example: سه شنبه ۱۷ خرداد ۱۳۹۰، ساعت ۰۸:۱۹:۲۳ (IRDT)
+				 */
 			case 'E':
+				tzset();
+				jalali_to_farsi(_l1, 10, 2, "۰", jtm->tm_hour);
+				jalali_to_farsi(_l2, 10, 2, "۰", jtm->tm_min);
+				jalali_to_farsi(_l3, 10, 2, "۰", jtm->tm_sec);
+				jalali_to_farsi(_la, 100, 2, "۰", jtm->tm_mday);
+				jalali_to_farsi(_lb, 100, 0, " ", jtm->tm_year);
+				snprintf(buf, MAX_BUF_SIZE, "%s %s %s %s، ساعت %s:%s:%s - %s",
+						 fa_jalali_days[jtm->tm_wday], _la,
+						 fa_jalali_months[jtm->tm_mon], _lb,
+						 _l1, _l2, _l3,
+						 (jtm->tm_zone == GMT_ZONE) ? GMT_ZONE_fa : 
+						 tzname_fa[jtm->tm_isdst]);
+
 				break;
 
-				/* Equivalent to %Y-%m-%d (the ISO 8601 date format). */
+				/*
+				 * Equivalent to %Y-%m-%d (similar to the ISO 8601 date format).
+				 */
 			case 'F':
 				snprintf(buf, MAX_BUF_SIZE, "%d-%02d-%02d", jtm->tm_year,
 						 jtm->tm_mon + 1, jtm->tm_mday);
 				break;
 
-				/* The abbreviated weekday name. (Persian-UTF8) */
+				/* The abbreviated weekday name. (Farsi-UTF8) */
 			case 'g':
 				strncpy(buf, fa_jalali_days_3[jtm->tm_wday], MAX_BUF_SIZE);
 				break;
 
-				/* The full weekday name. (Persian-UTF8) */
+				/* The full weekday name. (Farsi-UTF8) */
 			case 'G':
 				strncpy(buf, fa_jalali_days[jtm->tm_wday], MAX_BUF_SIZE);
 				break;
 
-				/* The abbreviated month name. (Persian-UTF8) */
+				/* The abbreviated month name. (Farsi-UTF8) */
 			case 'v':
 				strncpy(buf, fa_jalali_months_3[jtm->tm_mon], MAX_BUF_SIZE);
 				break;
 
-				/* The full month name. (Persian-UTF8) */
+				/* The full month name. (Farsi-UTF8) */
 			case 'V':
 				strncpy(buf, fa_jalali_months[jtm->tm_mon], MAX_BUF_SIZE);
 				break;
 
-				/* The abbreviated weekday name. (Persian) */
+				/* The abbreviated weekday name. (Farsi) */
 			case 'h':
 				strncpy(buf, jalali_days_3_fa[jtm->tm_wday], MAX_BUF_SIZE);
 				break;
 
-				/* The full weekday name. (Persian) */
+				/* The full weekday name. (Farsi) */
 			case 'q':
 				strncpy(buf, jalali_days_fa[jtm->tm_wday], MAX_BUF_SIZE);
 				break;
@@ -401,8 +437,14 @@ jstrftime(char* s, size_t max, const char* format, const struct jtm* jtm) {
 				snprintf(buf, MAX_BUF_SIZE, "\n");
 				break;
 
-				/* Unused. */
+				/*
+				 * Either "ق.ظ" or "ب.ظ" according to the given time value.
+				 * Noon is treated as "ق.ظ" and midnight as "ب.ظ".
+				 */
 			case 'O':
+				snprintf(buf, MAX_BUF_SIZE, "%s",
+						 (jtm->tm_hour >= 0 && jtm->tm_hour < 12) ?
+						 "ق.ظ" : "ب.ظ");
 				break;
 
 				/*
@@ -489,14 +531,44 @@ jstrftime(char* s, size_t max, const char* format, const struct jtm* jtm) {
 				snprintf(buf, MAX_BUF_SIZE, "%02d", tmp);
 				break;
 
+				/*
+				 * The day of the week as a decimal, range 0 to 6
+				 * Saturday being 0.  See also %u.
+				 */
+			case 'w':
+				snprintf(buf, MAX_BUF_SIZE, "%d", jtm->tm_wday);
+				break;
+
+				/*
+				 * The preferred date representation without the time
+				 * in Farsi. (utf8)
+				 */
+			case 'W':
+				jalali_to_farsi(_la, 100, 0, " ", jtm->tm_year);
+				jalali_to_farsi(_l1, 10, 2, "۰", jtm->tm_mon+1);
+				jalali_to_farsi(_l2, 10, 2, "۰", jtm->tm_mday);
+
+				snprintf(buf, MAX_BUF_SIZE, "%s/%s/%s", _la,
+						 _l1, _l2);
+				break;
+
 				/* The preferred date representation without the time. */
 			case 'x':
 				snprintf(buf, MAX_BUF_SIZE, "%02d/%02d/%d",
 						 jtm->tm_mday, jtm->tm_mon+1, jtm->tm_year);
 				break;
 
-				/* Unused. */
+				/*
+				 * The preferred time representation in Farsi. (utf8)
+				 */
 			case 'X':
+				jalali_to_farsi(_l1, 10, 2, "۰", jtm->tm_hour);
+				jalali_to_farsi(_l2, 10, 2, "۰", jtm->tm_min);
+				jalali_to_farsi(_l3, 10, 2, "۰", jtm->tm_sec);
+
+				snprintf(buf, MAX_BUF_SIZE, "%s:%s:%s",
+						 _l1, _l2, _l3);
+
 				break;
 
 				/*
@@ -551,6 +623,11 @@ jstrftime(char* s, size_t max, const char* format, const struct jtm* jtm) {
 				rb++;
 			}
 			i++;
+			_la[0] = 0;
+			_lb[0] = 0;
+			_l1[0] = 0;
+			_l2[0] = 0;
+			_l3[0] = 0;
 		}
 	}
 	s[rb] = '\0';
@@ -573,6 +650,9 @@ jstrptime(const char* s, const char* format, struct jtm* jtm) {
 	int i, j, k, f, c = 0;
 	char fd;
 
+	struct jtm _j;
+	time_t t;
+
 	s_s = strlen(s);
 	fmt_s = strlen(format);
 
@@ -599,7 +679,7 @@ jstrptime(const char* s, const char* format, struct jtm* jtm) {
 
 		/* Malformed string or format. */
 		if (format[j] != '%') {
-			return (char*) &s[i];
+			return NULL;
 		}
 
 		if (j+2 >= fmt_s) {
@@ -620,7 +700,7 @@ jstrptime(const char* s, const char* format, struct jtm* jtm) {
 
 			/* Delimiter not found in string. */
 			if (!pos_e) {
-				return (char*) &s[i];
+				return NULL;
 			}
 
 			diff1 = (pos_e - &s[i]);
@@ -696,6 +776,13 @@ jstrptime(const char* s, const char* format, struct jtm* jtm) {
 			jtm->tm_min = atoi(buf);
 			break;
 
+			/* Seconds since epoch. (1970/1/1) */
+		case 's':
+			t = (time_t) atol(buf);
+			jlocaltime_r(&t, &_j);
+			memcpy(jtm, &_j, sizeof(struct jtm));
+			break;
+
 			/* The second as a decimal number (range 00 to 59). */
 		case 'S':
 			jtm->tm_sec = atoi(buf);
@@ -718,7 +805,7 @@ jstrptime(const char* s, const char* format, struct jtm* jtm) {
 			jtm->tm_year = atoi(buf);
 			break;
 
-			/* The abbreviated or full weekday name. (Persian) */
+			/* The abbreviated or full weekday name. (Farsi) */
 		case 'q':
 		case 'h':
 			ptr = (fd == 'h') ? (char**) jalali_days_3_fa :
@@ -789,3 +876,44 @@ jctime_r(const time_t* timep, char* buf) {
 
 	return in_buf;
 }
+
+/*
+ * @Utils
+ * Utility functions for internal use.
+ * jalali_to_farsi() converts an integer's digits to Arabic-Indic
+ * padding works just like printf() field width.
+ */
+int jalali_to_farsi(char* buf, size_t n, int padding, char* pad,
+					int d) {
+	char _buf[100] = {0};
+	int i=0, j=0;
+	int p = 0;
+	int c = 0;
+	int cw = (pad[0] < 0) ? 2 : 1;
+
+	for (i=d; i!=0; c++,
+			 _buf[p] = farsi_digits[i%10 > 0 ? i%10 : -(i%10)][1],
+			 _buf[p+1] = farsi_digits[i%10 > 0 ? i%10 : -(i%10)][0],
+			 i/=10,
+			 p+=2);
+
+	if (d < 0) {
+		_buf[p] = '-';
+		c++;
+		p++;
+	}
+
+	_buf[p]= 0;
+	buf[0] = 0;
+	i=0;
+
+	for (i=0; (i<(padding - c)) && (i*cw < (int)(n-1));
+		 strcat(buf, pad), i++);
+	buf[i*cw] = 0;
+
+	for (j=0, i*=cw; (j<p) && (i < (int)(n-1));
+		 buf[i] = _buf[p - j - 1], i++, j++);
+	buf[i] = 0;
+
+	return i;
+}
